/**
 * Copyright 2005 The Apache Software Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.nutch.mime;

// JDK imports
import java.net.URL;
import java.io.File;
import java.io.IOException;
import java.io.DataInputStream;

// JUnit imports
import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import junit.textui.TestRunner;


/**
 * JUnit based tests for class {@link org.apache.nutch.mime.MimeTypes}
 *
 * @author Jerome Charron - http://frutch.free.fr/
 */
public class MimeTypesTest extends TestCase {
    
    public MimeTypesTest(String testName) {
        super(testName);
    }

    public static Test suite() {
        return new TestSuite(MimeTypesTest.class);
    }

    
    /**
     * Test of <code>getMimeType(String)</code>, <code>getMimeType(File)</code>
     * and <code>getMimeType(URL)</code> methods.
     */
    public void testGetMimeFromExtension() {

        try {
            DataInputStream in = new DataInputStream(
                            this.getClass().getResourceAsStream("mime-types.txt"));
            while(in.available() != 0) {
                String[] tokens = in.readLine().split(";");
                if (tokens[1].equals("rpm")) {
                    // TODO...
                } else if (!tokens[1].equals("")) {
                    assertEquals(tokens[0], MimeTypes.getMimeType("filename." + tokens[1]));
                    assertEquals(tokens[0], MimeTypes.getMimeType(new File("filename." + tokens[1])));
                    assertEquals(tokens[0], MimeTypes.getMimeType(new URL("http://frutch.free.fr/filename." + tokens[1])));
                }
            }
            in.close();
            assertNull(MimeTypes.getMimeType("filename.not-registered"));
            assertNull(MimeTypes.getMimeType(new File("filename.not-registered")));
            assertNull(MimeTypes.getMimeType(new URL("http://frutch.free.fr/filename.not-registered")));
        } catch(Exception e) {
            fail(e.toString());
        }
    }

    /** Test of <code>getMimeType(byte[])</code> method. */
    public void testGetMimeTypeFromData() {

        try {
            DataInputStream in = new DataInputStream(
                            this.getClass().getResourceAsStream("mime-types.txt"));
            while(in.available() != 0) {
                String[] tokens = in.readLine().split(";");
                if (tokens.length == 3) {
                    System.out.println("Testing... " + tokens[0] + "/" + tokens[2] + 
                                       " => " + MimeTypes.getMimeType(getData(tokens[2])));
                    assertEquals(tokens[0], MimeTypes.getMimeType(getData(tokens[2])));
                }
            }
            in.close();
        } catch(Exception e) {
            fail(e.toString());
        }
    }

    /** Test of <code>getMimeType(String, byte[])</code> method. */
    public void testGetMimeTypeFromDataAndExtension() {

        try {
            DataInputStream in = new DataInputStream(
                            this.getClass().getResourceAsStream("mime-types.txt"));
            while(in.available() != 0) {
                byte[] data = null;
                String[] tokens = in.readLine().split(";");
                if (tokens[1].equals("rpm")) {
                    // TODO...
                } else {
                    if (tokens.length == 3) {
                        data = getData(tokens[2]);
                    }
                    assertEquals(tokens[0], MimeTypes.getMimeType("filename." + tokens[1], data));
                }
            }
            in.close();
        } catch(Exception e) {
            fail(e.toString());
        }
    }
    
    private byte[] getData(String filename) throws IOException {
        byte[] data = new byte[MimeTypes.getMinLength()];
        DataInputStream in = new DataInputStream(this.getClass().getResourceAsStream(filename));
        in.read(data);
        in.close();
        return data;
    }
    
    /** Test of <code>add(MimeType)</code> method. */
    public void testAdd() {
        // TODO
    }


    public void main(String[] args) {
        TestRunner.run(suite());
    }
    
}
